@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2010 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -40,16 +40,27 @@ int HashDictionary(const char *string);
 static FILE *f_log = NULL;
 extern char *dir_dictionary;
 
-int linenum;
+extern char word_phonemes[N_WORD_PHONEMES];    // a word translated into phoneme codes
+
+static int linenum;
 static int error_count;
-static int transpose_offset;  // transpose character range for LookupDictList()
-static int transpose_min;
-static int transpose_max;
 static int text_mode = 0;
 static int debug_flag = 0;
+static int error_need_dictionary = 0;
 
-int hash_counts[N_HASH_DICT];
-char *hash_chains[N_HASH_DICT];
+static int hash_counts[N_HASH_DICT];
+static char *hash_chains[N_HASH_DICT];
+static char letterGroupsDefined[N_LETTER_GROUPS];
+
+MNEM_TAB mnem_rules[] = {
+	{"unpr",   0x01},
+	{"w_alt2", 0x12},
+	{"w_alt3", 0x13},
+	{"w_alt", 0x11},   // note: put longer names before their sub-strings
+	{"p_alt2", 0x22},
+	{"p_alt3", 0x23},
+	{"p_alt", 0x21},
+	{NULL, -1} };
 
 MNEM_TAB mnem_flags[] = {
 	// these in the first group put a value in bits0-3 of dictionary_flags
@@ -78,17 +89,23 @@ MNEM_TAB mnem_flags[] = {
 	{"$strend2",   12},    /* full stress if at end of clause, or only followed by unstressed */
 	{"$unstressend",13},   /* reduce stress at end of clause */
 	{"$atend",     14},    /* use this pronunciation if at end of clause */
+	{"$atstart",   15},   // use this pronunciation at start of clause
 
-	{"$dot",       16},   /* ignore '.' after this word (abbreviation) */
 	{"$abbrev",    17},    /* use this pronuciation rather than split into letters */
 	{"$stem",      18},   // must have a suffix
 
 // language specific
 	{"$double",    19},   // IT double the initial consonant of next word
 	{"$alt",       20},   // use alternative pronunciation
+	{"$alt1",      20},   // synonym for $alt
 	{"$alt2",      21},
-	
+	{"$alt3",      22},
+	{"$combine",   23},   // Combine with the next word
 
+	{"$dot",       24},   // ignore '.' after this word (abbreviation)
+	{"$hasdot",    25},   // use this pronunciation if there is a dot after the word
+
+	{"$max3",      27},   // limit to 3 repetitions
 	{"$brk",       28},   // a shorter $pause
 	{"$text",      29},   // word translates to replcement text, not phonemes
 
@@ -104,6 +121,7 @@ MNEM_TAB mnem_flags[] = {
 	{"$capital", 0x29},   /* use this pronunciation if initial letter is upper case */
 	{"$allcaps", 0x2a},   /* use this pronunciation if initial letter is upper case */
 	{"$accent",  0x2b},   // character name is base-character name + accent name
+	{"$sentence",0x2d},   // only if this clause is a sentence (i.e. terminator is {. ? !} not {, ; :}
 
 	// doesn't set dictionary_flags
 	{"$?",        100},   // conditional rule, followed by byte giving the condition number
@@ -114,10 +132,13 @@ MNEM_TAB mnem_flags[] = {
 };
 
 
+#define LEN_GROUP_NAME  12
+
 typedef struct {
-	char name[6];
+	char name[LEN_GROUP_NAME+1];
 	unsigned int start;
 	unsigned int length;
+	int group3_ix;
 } RGROUP;
 
 
@@ -132,6 +153,7 @@ int isspace2(unsigned int c)
 }
 
 
+
 static FILE *fopen_log(const char *fname,const char *access)
 {//==================================================
 // performs fopen, but produces error message to f_log if it fails
@@ -146,9 +168,8 @@ static FILE *fopen_log(const char *fname,const char *access)
 }
 
 
-#ifdef OPT_FORMAT
-static const char *lookup_mnem(MNEM_TAB *table, int value)
-//========================================================
+const char *LookupMnemName(MNEM_TAB *table, const int value)
+//==========================================================
 /* Lookup a mnemonic string in a table, return its name */
 {
    while(table->mnem != NULL)
@@ -157,15 +178,196 @@ static const char *lookup_mnem(MNEM_TAB *table, int value)
          return(table->mnem);
       table++;
    }
-   return("??");   /* not found */
-}   /* end of mnem */
-#endif
+   return("");   /* not found */
+}   /* end of LookupMnemValue */
+
+
+char *print_dictionary_flags(unsigned int *flags)
+{//==============================================
+	static char buf[20];
+
+	sprintf(buf,"%s  0x%x/%x",LookupMnemName(mnem_flags,(flags[0] & 0xf)+0x40), flags[0], flags[1]);
+	return(buf);
+}
 
 
 
 
-int compile_line(char *linebuf, char *dict_line, int *hash)
-{//========================================================
+char *DecodeRule(const char *group_chars, int group_length, char *rule, int control)
+{//=================================================================================
+/* Convert compiled match template to ascii */
+
+   unsigned char rb;
+	unsigned char c;
+	char *p;
+   int  ix;
+	int  match_type;
+	int  finished=0;
+	int  value;
+	int  linenum=0;
+	int  flags;
+	int  suffix_char;
+	int  condition_num=0;
+	int  at_start = 0;
+	const char *name;
+   char buf[60];
+   char buf_pre[60];
+	char suffix[20];
+	static char output[60];
+
+	static char symbols[] = {' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
+		'&','%','+','#','S','D','Z','A','L','!',' ','@','?','J','N','K','V','?','T','X','?','W'};
+
+	static char symbols_lg[] = {'A','B','C','H','F','G','Y'};
+
+	match_type = 0;
+   buf_pre[0] = 0;
+
+	for(ix=0; ix<group_length; ix++)
+	{
+		buf[ix] = group_chars[ix];
+	}
+	buf[ix] = 0;
+
+	p = &buf[strlen(buf)];
+   while(!finished)
+   {
+		rb = *rule++;
+
+		if(rb <= RULE_LINENUM)
+		{
+			switch(rb)
+			{
+			case 0:
+			case RULE_PHONEMES:
+				finished=1;
+				break;
+			case RULE_PRE_ATSTART:
+				at_start = 1;  // drop through to next case
+			case RULE_PRE:
+				match_type = RULE_PRE;
+				*p = 0;
+				p = buf_pre;
+				break;
+			case RULE_POST:
+				match_type = RULE_POST;
+				*p = 0;
+				strcat(buf," (");
+				p = &buf[strlen(buf)];
+				break;
+			case RULE_PH_COMMON:
+				break;
+			case RULE_CONDITION:
+				/* conditional rule, next byte gives condition number */
+				condition_num = *rule++;
+				break;
+			case RULE_LINENUM:
+				value = (rule[1] & 0xff) - 1;
+				linenum = (rule[0] & 0xff) - 1 + (value * 255);
+				rule+=2;
+				break;
+			}
+			continue;
+		}
+		
+		if(rb == RULE_DOLLAR)
+		{
+			value = *rule++ & 0xff;
+			if((value != 0x01) || (control & FLAG_UNPRON_TEST))
+			{
+				p[0] = '$';
+				name = LookupMnemName(mnem_rules, value);
+				strcpy(&p[1],name);
+				p += (strlen(name)+1);
+			}
+			c = ' ';
+		}
+		else
+		if(rb == RULE_ENDING)
+		{
+			static const char *flag_chars = "eipvdfq tba ";
+			flags = ((rule[0] & 0x7f)<< 8) + (rule[1] & 0x7f);
+			suffix_char = 'S';
+			if(flags & (SUFX_P >> 8))
+				suffix_char = 'P';
+			sprintf(suffix,"%c%d",suffix_char,rule[2] & 0x7f);
+			rule += 3;
+			for(ix=0;ix<9;ix++)
+			{
+				if(flags & 1)
+					sprintf(&suffix[strlen(suffix)],"%c",flag_chars[ix]);
+				flags = (flags >> 1);
+			}
+			strcpy(p,suffix);
+			p += strlen(suffix);
+			c = ' ';
+		}
+		else
+		if(rb == RULE_LETTERGP)
+		{
+			c = symbols_lg[*rule++ - 'A'];
+		}
+		else
+		if(rb == RULE_LETTERGP2)
+		{
+			value = *rule++ - 'A';
+			p[0] = 'L';
+			p[1] = (value / 10) + '0';
+			c = (value % 10) + '0';
+
+			if(match_type == RULE_PRE)
+			{
+				p[0] = c;
+				c = 'L';
+			}
+			p+=2;
+		}
+		else
+		if(rb <= RULE_LAST_RULE)
+			c = symbols[rb];
+		else
+		if(rb == RULE_SPACE)
+			c = '_';
+		else
+			c = rb;
+		*p++ = c;
+	}
+	*p = 0;
+
+	p = output;
+	if(linenum > 0)
+	{
+		sprintf(p,"%5d:\t",linenum);
+		p += 7;
+	}
+	if(condition_num > 0)
+	{
+		sprintf(p,"?%d ",condition_num);
+		p = &p[strlen(p)];
+	}
+	if(((ix = strlen(buf_pre)) > 0) || at_start)
+	{
+		if(at_start)
+			*p++ = '_';
+		while(--ix >= 0)
+			*p++ = buf_pre[ix];
+		*p++ = ')';
+		*p++ = ' ';
+	}
+	*p = 0;
+	strcat(p,buf);
+	ix = strlen(output);
+	while(ix < 8)
+		output[ix++]=' ';
+	output[ix]=0;
+   return(output);
+}   /* end of DecodeRule */
+
+
+
+
+static int compile_line(char *linebuf, char *dict_line, int *hash)
+{//===============================================================
 // Compile a line in the language_list file
 	unsigned char  c;
 	char *p;
@@ -177,6 +379,7 @@ int compile_line(char *linebuf, char *dict_line, int *hash)
 	int  flag_offset;
 	int  length;
 	int  multiple_words = 0;
+	int  multiple_numeric_hyphen = 0;
 	char *multiple_string = NULL;
 	char *multiple_string_end = NULL;
 	
@@ -184,6 +387,7 @@ int compile_line(char *linebuf, char *dict_line, int *hash)
 	int len_phonetic;
 	int text_not_phonemes;   // this word specifies replacement text, not phonemes
 	unsigned int  wc;
+	int all_upper_case;
 	
 	char *mnemptr;
 	char *comment;
@@ -196,6 +400,10 @@ static char nullstring[] = {0};
 	text_not_phonemes = 0;
 	phonetic = word = nullstring;
 
+if(memcmp(linebuf,"_-",2)==0)
+{
+step=1;  // TEST
+}
 	p = linebuf;
 //	while(isspace2(*p)) p++;
 
@@ -310,6 +518,18 @@ static char nullstring[] = {0};
 			break;
 	
 		case 1:
+			if((c == '-') && multiple_words)
+			{
+				if(isdigit(word[0]))
+				{
+					multiple_numeric_hyphen = 1;
+				}
+				else
+				{
+					flag_codes[n_flag_codes++] = BITNUM_FLAG_HYPHENATED;
+				}
+				c = ' ';
+			}
 			if(isspace2(c))
 			{
 				p[0] = 0;   /* terminate english word */
@@ -383,15 +603,36 @@ static char nullstring[] = {0};
 	if(text_mode)
 		text_not_phonemes = 1;
 
-	if(text_not_phonemes != translator->langopts.textmode)
-	{
-		flag_codes[n_flag_codes++] = BITNUM_FLAG_TEXTMODE;
-	}
-
 	if(text_not_phonemes)
 	{
-		// this is replacement text, so don't encode as phonemes. Restrict the length of the replacement word
-		strncpy0(encoded_ph,phonetic,N_WORD_BYTES-4);
+		if(word[0] == '_')
+		{
+			// This is a special word, used by eSpeak.  Translate this into phonemes now
+			strcat(phonetic, " ");     // need a space to indicate word-boundary
+
+	// PROBLEM  vowel reductions are not applied to the translated phonemes
+	// condition rules are not applied
+			TranslateWord(translator,phonetic,0,NULL);
+			text_not_phonemes = 0;
+			strncpy0(encoded_ph, word_phonemes, N_WORD_BYTES-4);
+
+			if((word_phonemes[0] == 0) && (error_need_dictionary < 3))
+			{
+				// the dictionary was not loaded, we need a second attempt
+				error_need_dictionary++;
+				fprintf(f_log,"%5d: Need to compile dictionary again\n",linenum);
+			}
+{
+//char decoded_phonemes[128];
+//DecodePhonemes(word_phonemes,decoded_phonemes);
+//printf("Translator %x  %s  [%s] [%s]\n",translator->translator_name,word,phonetic,decoded_phonemes);
+}
+		}
+		else
+		{
+			// this is replacement text, so don't encode as phonemes. Restrict the length of the replacement word
+			strncpy0(encoded_ph,phonetic,N_WORD_BYTES-4);
+		}
 	}
 	else
 	{
@@ -416,6 +657,12 @@ static char nullstring[] = {0};
 		}
 	}
 
+	if(text_not_phonemes != translator->langopts.textmode)
+	{
+		flag_codes[n_flag_codes++] = BITNUM_FLAG_TEXTMODE;
+	}
+
+
 	if(sscanf(word,"U+%x",&wc) == 1)
 	{
 		// Character code
@@ -423,18 +670,41 @@ static char nullstring[] = {0};
 		word[ix] = 0;
 	}
 	else
-	if((word[0] & 0x80)==0)  // 7 bit ascii only
+	if(word[0] != '_')
 	{
-		// If first letter is uppercase, convert to lower case.  (Only if it's 7bit ascii)
-		// ??? need to consider utf8 here
-		word[0] = tolower(word[0]);
+		// convert to lower case, and note if the word is all-capitals
+		int c2;
+
+		all_upper_case = 1;
+		p = word;
+		for(p=word;;)
+		{
+			// this assumes that the lower case char is the same length as the upper case char
+			// OK, except for Turkish "I", but use towlower() rather than towlower2()
+			ix = utf8_in(&c2,p);
+			if(c2 == 0)
+				break;
+			if(iswupper(c2))
+			{
+				utf8_out(towlower(c2),p);
+			}
+			else
+			{
+				all_upper_case = 0;
+			}
+			p += ix;
+		}
+		if(all_upper_case)
+		{
+			flag_codes[n_flag_codes++] = BITNUM_FLAG_ALLCAPS;
+		}
 	}
 
 	len_word = strlen(word);
 
-	if(transpose_offset > 0)
+	if(translator->transpose_min > 0)
 	{
-		len_word = TransposeAlphabet(word, transpose_offset, transpose_min, transpose_max);
+		len_word = TransposeAlphabet(translator, word);
 	}
 
 	*hash = HashDictionary(word);
@@ -473,6 +743,10 @@ static char nullstring[] = {0};
 		{
 			dict_line[length++] = 80 + multiple_words;
 			ix = multiple_string_end - multiple_string;
+			if(multiple_numeric_hyphen)
+			{
+				dict_line[length++] = ' ';
+			}
 			memcpy(&dict_line[length],multiple_string,ix);
 			length += ix;
 		}
@@ -514,8 +788,8 @@ static char nullstring[] = {0};
 
 
 
-void compile_dictlist_start(void)
-{//==============================
+static void compile_dictlist_start(void)
+{//=====================================
 // initialise dictionary list
 	int ix;
 	char *p;
@@ -536,8 +810,8 @@ void compile_dictlist_start(void)
 }
 
 
-void compile_dictlist_end(FILE *f_out)
-{//===================================
+static void compile_dictlist_end(FILE *f_out)
+{//==========================================
 // Write out the compiled dictionary list
 	int hash;
 	int length;
@@ -573,8 +847,8 @@ void compile_dictlist_end(FILE *f_out)
 
 
 
-int compile_dictlist_file(const char *path, const char* filename)
-{//==============================================================
+static int compile_dictlist_file(const char *path, const char* filename)
+{//=====================================================================
 	int  length;
 	int  hash;
 	char *p;
@@ -586,9 +860,14 @@ int compile_dictlist_file(const char *path, const char* filename)
 	
 	text_mode = 0;
 
-	sprintf(fname,"%s%s",path,filename);
+	// try with and without '.txt' extension
+	sprintf(fname,"%s%s.txt",path,filename);
 	if((f_in = fopen(fname,"r")) == NULL)
-		return(-1);
+	{
+		sprintf(fname,"%s%s",path,filename);
+		if((f_in = fopen(fname,"r")) == NULL)
+			return(-1);
+	}
 
 	fprintf(f_log,"Compiling: '%s'\n",fname);
 
@@ -627,26 +906,20 @@ int compile_dictlist_file(const char *path, const char* filename)
 
 
 
-char rule_cond[80];
-char rule_pre[80];
-char rule_post[80];
-char rule_match[80];
-char rule_phonemes[80];
-char group_name[12];
+static char rule_cond[80];
+static char rule_pre[80];
+static char rule_post[80];
+static char rule_match[80];
+static char rule_phonemes[80];
+static char group_name[LEN_GROUP_NAME+1];
+static int group3_ix;
 
 #define N_RULES 2000		// max rules for each group
 
 
-int hexdigit(char c)
-{//=================
-	if(isdigit(c))
-		return(c - '0');
-	return(tolower(c) - 'a' + 10);
-}
 
-
-void copy_rule_string(char *string, int &state)
-{//============================================
+static void copy_rule_string(char *string, int &state)
+{//===================================================
 // state 0: conditional, 1=pre, 2=match, 3=post, 4=phonemes
 	static char *outbuf[5] = {rule_cond, rule_pre, rule_match, rule_post, rule_phonemes};
 	static int next_state[5] = {2,2,4,4,4};
@@ -658,6 +931,7 @@ void copy_rule_string(char *string, int &state)
 	int  sxflags;
 	int  value;
 	int  literal;
+	MNEM_TAB *mr;
 
 	if(string[0] == 0) return;
 
@@ -754,7 +1028,8 @@ void copy_rule_string(char *string, int &state)
 					c = RULE_CAPITAL;
 					break;
 				case 'T':
-					c = RULE_ALT1;
+					output[ix++] = RULE_DOLLAR;
+					c = 0x11;
 					break;
 				case 'W':
 					c = RULE_SPELLING;
@@ -762,15 +1037,24 @@ void copy_rule_string(char *string, int &state)
 				case 'X':
 					c = RULE_NOVOWELS;
 					break;
+				case 'J':
+					c = RULE_SKIPCHARS;
+					break;
 				case 'L':
 					// expect two digits
 					c = *p++ - '0';
 					value = *p++ - '0';
 					c = c * 10 + value;
-					if((value < 0) || (value > 9) || (c <= 0) || (c >= N_LETTER_GROUPS))
+					if((value < 0) || (value > 9))
 					{
 						c = 0;
-						fprintf(f_log,"%5d: Expected 2 digits after 'L'",linenum);
+						fprintf(f_log,"%5d: Expected 2 digits after 'L'\n",linenum);
+						error_count++;
+					}
+					else
+					if((c <= 0) || (c >= N_LETTER_GROUPS) || (letterGroupsDefined[(int)c] == 0))
+					{
+						fprintf(f_log,"%5d: Letter group L%.2d not defined\n",linenum,c);
 						error_count++;
 					}
 					c += 'A';
@@ -786,9 +1070,30 @@ void copy_rule_string(char *string, int &state)
 					}
 					break;
 
+				case '$':
+					output[ix++] = RULE_DOLLAR;
+					c = 0;
+					mr = mnem_rules;
+					while(mr->mnem != NULL)
+					{
+						len = strlen(mr->mnem);
+						if(memcmp(p, mr->mnem, len) == 0)
+						{
+							c = mr->value;
+							p += len;
+							break;
+						}
+						mr++;
+					}
+					if(c == 0)
+					{
+						fprintf(f_log,"%5d: $ command not recognized\n",linenum);
+						error_count++;
+					}
+					break;
+
 				case 'P':
 					sxflags |= SUFX_P;   // Prefix, now drop through to Suffix
-				case '$':   // obsolete, replaced by S
 				case 'S':
 					output[ix++] = RULE_ENDING;
 					value = 0;
@@ -823,6 +1128,9 @@ void copy_rule_string(char *string, int &state)
 						case 'b':
 							sxflags |= SUFX_B;
 							break;
+						case 'a':
+							sxflags |= SUFX_A;
+							break;
 						default:
 							if(isdigit(c))
 								value = (value*10) + (c - '0');
@@ -846,8 +1154,8 @@ void copy_rule_string(char *string, int &state)
 
 
 
-char *compile_rule(char *input)
-{//============================
+static char *compile_rule(char *input)
+{//===================================
 	int ix;
 	unsigned char c;
 	int wc;
@@ -855,6 +1163,7 @@ char *compile_rule(char *input)
 	char *prule;
 	int len;
 	int len_name;
+	int start;
 	int state=2;
 	int finish=0;
 	int pre_bracket=0;
@@ -891,6 +1200,11 @@ char *compile_rule(char *input)
 			copy_rule_string(buf,state);
 			state = 3;
 			p = buf;
+			if(input[ix+1] == ' ')
+			{
+				fprintf(f_log,"%5d: Syntax error. Space after (\n",linenum);
+				error_count++;
+			}
 			break;
 			
 		case '\n':		// end of line
@@ -925,7 +1239,14 @@ char *compile_rule(char *input)
 		strcpy(rule_match,group_name);
 
 	if(rule_match[0]==0)
+	{
+		if(rule_post[0] != 0)
+		{
+			fprintf(f_log,"%5d: Syntax error\n",linenum);
+			error_count++;
+		}
 		return(NULL);
+	}
 
 	EncodePhonemes(rule_phonemes,buf,bad_phoneme);
 	for(ix=0;; ix++)
@@ -933,7 +1254,7 @@ char *compile_rule(char *input)
 		if((c = buf[ix])==0) break;
 		if(c==255)
 		{
-			fprintf(f_log,"%5d: Bad phoneme [%c] in %s",linenum,bad_phoneme[0],input);
+			fprintf(f_log,"%5d: Bad phoneme [%c] in %s\n",linenum,bad_phoneme[0],input);
 			error_count++;
 			break;
 		}
@@ -944,7 +1265,7 @@ char *compile_rule(char *input)
 	len_name = strlen(group_name);
 	if((len_name > 0) && (memcmp(rule_match,group_name,len_name) != 0))
 	{
-		utf8_in(&wc,rule_match,0);
+		utf8_in(&wc,rule_match);
 		if((group_name[0] == '9') && IsDigit(wc))
 		{
 			// numeric group, rule_match starts with a digit, so OK
@@ -993,9 +1314,21 @@ char *compile_rule(char *input)
 	}
 	if(rule_pre[0] != 0)
 	{
-		output[len++] = RULE_PRE;
+		start = 0;
+		if(rule_pre[0] == RULE_SPACE)
+		{
+			// omit '_' at the beginning of the pre-string and imply it by using RULE_PRE_ATSTART
+			c = RULE_PRE_ATSTART;
+			start = 1;
+		}
+		else
+		{
+			c = RULE_PRE;
+		}
+		output[len++] = c;
+
 		// output PRE string in reverse order
-		for(ix = strlen(rule_pre)-1; ix>=0; ix--)
+		for(ix = strlen(rule_pre)-1; ix>=start; ix--)
 			output[len++] = rule_pre[ix];
 	}
 
@@ -1011,8 +1344,8 @@ char *compile_rule(char *input)
 }  //  end of compile_rule
 
 
-static int __cdecl string_sorter(char **a, char **b)
-{//=================================================
+int __cdecl string_sorter(char **a, char **b)
+{//===========================================
 	char *pa, *pb;
 	int ix;
 
@@ -1026,7 +1359,10 @@ static int __cdecl string_sorter(char **a, char **b)
 
 static int __cdecl rgroup_sorter(RGROUP *a, RGROUP *b)
 {//===================================================
+// Sort long names before short names
 	int ix;
+	ix = strlen(b->name) - strlen(a->name);
+	if(ix != 0) return(ix);
 	ix = strcmp(a->name,b->name);
 	if(ix != 0) return(ix);
 	return(a->start-b->start);
@@ -1034,8 +1370,8 @@ static int __cdecl rgroup_sorter(RGROUP *a, RGROUP *b)
 
 
 #ifdef OUTPUT_FORMAT
-void print_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
-{//======================================================================
+static void print_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
+{//=============================================================================
 	int rule;
 	int ix;
 	unsigned char c;
@@ -1141,8 +1477,8 @@ void print_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
 
 
 //#define LIST_GROUP_INFO
-void output_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
-{//=======================================================================
+static void output_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
+{//==============================================================================
 	int ix;
 	int len1;
 	int len2;
@@ -1210,36 +1546,77 @@ void output_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
 static int compile_lettergroup(char *input, FILE *f_out)
 {//=====================================================
 	char *p;
+	char *p_start;
 	int group;
+	int ix;
+	int n_items;
+	int length;
+	int max_length = 0;
+
+	#define N_LETTERGP_ITEMS 200
+	char *items[N_LETTERGP_ITEMS];
+	char item_length[N_LETTERGP_ITEMS];
 
 	p = input;
 	if(!isdigit(p[0]) || !isdigit(p[1]))
 	{
+		fprintf(f_log,"%5d: Expected 2 digits after '.L'\n",linenum);
+		error_count++;
 		return(1);
 	}
 
-	group = atoi(&p[1]);
+	group = atoi(&p[0]);
 	if(group >= N_LETTER_GROUPS)
+	{
+		fprintf(f_log,"%5d: lettergroup out of range (01-%.2d)\n",linenum,N_LETTER_GROUPS-1);
+		error_count++;
 		return(1);
+	}
 
 	while(!isspace2(*p)) p++;
 
 	fputc(RULE_GROUP_START,f_out);
 	fputc(RULE_LETTERGP2,f_out);
 	fputc(group + 'A', f_out);
+	if(letterGroupsDefined[group] != 0)
+	{
+		fprintf(f_log,"%5d: lettergroup L%.2d is already defined\n",linenum,group);
+		error_count++;
+	}
+	letterGroupsDefined[group] = 1;
 
-	for(;;)
+	n_items = 0;
+	while(n_items < N_LETTERGP_ITEMS)
 	{
 		while(isspace2(*p)) p++;
 		if(*p == 0)
 			break;
-		
+
+		items[n_items] = p_start = p;
 		while((*p & 0xff) > ' ')
 		{
-			fputc(*p++, f_out);
+			p++;
 		}
-		fputc(0,f_out);
+		*p++ = 0;
+		length = p - p_start;
+		if(length > max_length)
+			max_length = length;
+		item_length[n_items++] = length;
 	}
+
+	// write out the items, longest first
+	while(max_length > 1)
+	{
+		for(ix=0; ix < n_items; ix++)
+		{
+			if(item_length[ix] == max_length)
+			{
+				fwrite(items[ix],1,max_length,f_out);
+			}
+		}
+		max_length--;
+	}
+
 	fputc(RULE_GROUP_END,f_out);
 
 	return(0);
@@ -1257,6 +1634,7 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 	int n_rules=0;
 	int count=0;
 	int different;
+	int wc;
 	const char *prev_rgroup_name;
 	unsigned int char_code;
 	int compile_mode=0;
@@ -1265,6 +1643,7 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 	char *rules[N_RULES];
 
 	int n_rgroups = 0;
+	int n_groups3 = 0;
 	RGROUP rgroup[N_RULE_GROUP2];
 	
 	linenum = 0;
@@ -1292,6 +1671,7 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 			if(n_rules > 0)
 			{
 				strcpy(rgroup[n_rgroups].name,group_name);
+				rgroup[n_rgroups].group3_ix = group3_ix;
 				rgroup[n_rgroups].start = ftell(f_temp);
 				output_rule_group(f_temp,n_rules,rules,group_name);
 				rgroup[n_rgroups].length = ftell(f_temp) - rgroup[n_rgroups].start;
@@ -1312,11 +1692,7 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 
 			if(memcmp(buf,".L",2)==0)
 			{
-				if(compile_lettergroup(&buf[2], f_out) != 0)
-				{
-					fprintf(f_log,"%5d: Bad lettergroup\n",linenum);
-					error_count++;
-				}
+				compile_lettergroup(&buf[2], f_out);
 				continue;
 			}
 
@@ -1338,10 +1714,11 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 				p = (unsigned char *)&buf[6];
 				while((p[0]==' ') || (p[0]=='\t')) p++;    // Note: Windows isspace(0xe1) gives TRUE !
 				ix = 0;
-				while((*p > ' ') && (ix<12))
+				while((*p > ' ') && (ix < LEN_GROUP_NAME))
 					group_name[ix++] = *p++;
 				group_name[ix]=0;
-	
+				group3_ix = 0;
+
 				if(sscanf(group_name,"0x%x",&char_code)==1)
 				{
 					// group character is given as a character code (max 16 bits)
@@ -1354,10 +1731,21 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 					*p++ = char_code;
 					*p = 0;
 				}
-	
-				if(strlen(group_name) > 2)
+				else
 				{
-					if(utf8_in(&c,group_name,0) < 2)
+					if(translator->letter_bits_offset > 0)
+					{
+						utf8_in(&wc, group_name);
+						if(((ix = (wc - translator->letter_bits_offset)) >= 0) && (ix < 128))
+						{
+							group3_ix = ix+1;   // not zero
+						}
+					}
+				}
+	
+				if((group3_ix == 0) && (strlen(group_name) > 2))
+				{
+					if(utf8_in(&c,group_name) < 2)
 					{
 						fprintf(f_log,"%5d: Group name longer than 2 bytes (UTF8)",linenum);
 						error_count++;
@@ -1393,7 +1781,7 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 				ix = 0;
 				while((unsigned char)(*p) > 0x20)   // not space or zero-byte
 				{
-					p += utf8_in(&c,p,0);
+					p += utf8_in(&c,p);
 					replace1 += (c << ix);
 					ix += 16;
 				}
@@ -1401,7 +1789,7 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 				ix = 0;
 				while((unsigned char)(*p) > 0x20)
 				{
-					p += utf8_in(&c,p,0);
+					p += utf8_in(&c,p);
 					replace2 += (c << ix);
 					ix += 16;
 				}
@@ -1433,7 +1821,17 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 			if(gp > 0)
 				fputc(RULE_GROUP_END,f_out);
 			fputc(RULE_GROUP_START,f_out);
-			fprintf(f_out, prev_rgroup_name = rgroup[gp].name);
+
+			if(rgroup[gp].group3_ix != 0)
+			{
+				n_groups3++;
+				fputc(1,f_out);
+				fputc(rgroup[gp].group3_ix, f_out);
+			}
+			else
+			{
+				fprintf(f_out, "%s", prev_rgroup_name = rgroup[gp].name);
+			}
 			fputc(0,f_out);
 		}
 
@@ -1453,13 +1851,12 @@ static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 	fclose(f_temp);
 	remove(fname_temp);
 
-	fprintf(f_log,"\t%d rules, %d groups\n\n",count,n_rgroups);
+	fprintf(f_log,"\t%d rules, %d groups (%d)\n\n",count,n_rgroups,n_groups3);
 	return(0);
 }  //  end of compile_dictrules
 
 
 
-
 int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, char *fname_err, int flags)
 {//=====================================================================================================
 // fname:  space to write the filename in case of error
@@ -1475,6 +1872,9 @@ int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, cha
 	char path[sizeof(path_home)+40];       // path_dsource+20
 
 	error_count = 0;
+	error_need_dictionary = 0;
+	memset(letterGroupsDefined,0,sizeof(letterGroupsDefined));
+
 	debug_flag = flags & 1;
 
 	if(dsource == NULL)
@@ -1485,14 +1885,18 @@ int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, cha
 	if(f_log == NULL)
 		f_log = stderr;
 
+	// try with and without '.txt' extension
 	sprintf(path,"%s%s_",dsource,dict_name);
-	sprintf(fname_in,"%srules",path);
-	f_in = fopen_log(fname_in,"r");
-	if(f_in == NULL)
+	sprintf(fname_in,"%srules.txt",path);
+	if((f_in = fopen(fname_in,"r")) == NULL)
 	{
-		if(fname_err)
-			strcpy(fname_err,fname_in);
-		return(-1);
+		sprintf(fname_in,"%srules",path);
+		if((f_in = fopen_log(fname_in,"r")) == NULL)
+		{
+			if(fname_err)
+				strcpy(fname_err,fname_in);
+			return(-1);
+		}
 	}
 
 	sprintf(fname_out,"%s%c%s_dict",path_home,PATHSEP,dict_name);
@@ -1504,24 +1908,13 @@ int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, cha
 	}
 	sprintf(fname_temp,"%s%ctemp",path_home,PATHSEP);
 
-	transpose_offset = 0;
-
-	if(strcmp(dict_name,"ru") == 0)
-	{
-		// transpose cyrillic alphabet from unicode to iso8859-5
-//		transpose_offset = 0x430-0xd0;
-		transpose_offset = 0x42f;   // range 0x01 to 0x22
-		transpose_min = 0x430;
-		transpose_max = 0x451;
-	}
-
 	value = N_HASH_DICT;
 	Write4Bytes(f_out,value);
 	Write4Bytes(f_out,offset_rules);
 
 	compile_dictlist_start();
 
-	fprintf(f_log,"Using phonemetable: '%s'\n",PhonemeTabName());
+	fprintf(f_log,"Using phonemetable: '%s'\n",phoneme_tab_list[phoneme_tab_number].name);
 	compile_dictlist_file(path,"roots");
 	if(translator->langopts.listx)
 	{
@@ -1547,7 +1940,7 @@ int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, cha
 	Write4Bytes(f_out,offset_rules);
 	fclose(f_out);
 
-	translator->LoadDictionary(dict_name,0);
+	LoadDictionary(translator, dict_name, 0);
 
 	return(error_count);
 }  //  end of compile_dictionary
