@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2010 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -49,8 +49,8 @@ MNEM_TAB genders [] = {
 	{"female", 2},
 	{NULL, 0 }};
 
-//int tone_points[10] = {250,140, 1200,110, -1,0, -1,0, -1,0};
-int tone_points[10] = {600,170, 1200,135, 2000,110, 3000,110, -1,0};
+int tone_points[12] = {600,170, 1200,135, 2000,110, 3000,110, -1,0};
+//int tone_points[12] = {250,200,  400,170, 600,170, 1200,135, 2000,110, -1,0};
 
 // limit the rate of change for each formant number
 //static int formant_rate_22050[9] = {50, 104, 165, 230, 220, 220, 220, 220, 220};  // values for 22kHz sample rate
@@ -69,51 +69,57 @@ static int len_path_voices;
 espeak_VOICE voice_selected;
 
 
-
-#define V_NAME       1
-#define V_LANGUAGE   2
-#define V_GENDER     3
-#define V_TRANSLATOR  4
-#define V_PHONEMES    5
-#define V_DICTIONARY  6
+enum {
+	V_NAME = 1,
+	V_LANGUAGE,
+	V_GENDER,
+	V_TRANSLATOR,
+	V_PHONEMES,
+	V_DICTIONARY,
 
 // these affect voice quality, are independent of language
-#define V_FORMANT     7
-#define V_PITCH       8
-#define V_ECHO        9
-#define V_FLUTTER    10
-#define V_ROUGHNESS  11
-#define V_CLARITY    12
-#define V_TONE       13
-#define V_VOICING    14
-#define V_BREATH     15
-#define V_BREATHW    16
+	V_FORMANT,
+	V_PITCH,
+	V_ECHO,
+	V_FLUTTER,
+	V_ROUGHNESS,
+	V_CLARITY,
+	V_TONE,
+	V_VOICING,
+	V_BREATH,
+	V_BREATHW,
 
 // these override defaults set by the translator
-#define V_WORDGAP    17
-#define V_INTONATION 18
-#define V_STRESSLENGTH  19
-#define V_STRESSAMP  20
-#define V_STRESSADD  21
-#define V_DICTRULES   22
-#define V_STRESSRULE  23
-#define V_CHARSET     24
-#define V_NUMBERS     25
-#define V_OPTION      26
+	V_WORDGAP,
+	V_INTONATION,
+	V_TUNES,
+	V_STRESSLENGTH,
+	V_STRESSAMP,
+	V_STRESSADD,
+	V_DICTRULES,
+	V_STRESSRULE,
+	V_CHARSET,
+	V_NUMBERS,
+	V_OPTION,
 
-#define V_MBROLA     27
+	V_MBROLA,
+	V_KLATT,
+	V_FAST,
+	V_SPEED,
 
 // these need a phoneme table to have been specified
-#define V_REPLACE    28
-#define V_CONSONANTS 29
+	V_REPLACE,
+	V_CONSONANTS
+};
 
 
-typedef struct {
-	const char *mnem;
-	int data;
-} keywtab_t;
 
-static keywtab_t keyword_tab[] = {
+static MNEM_TAB options_tab[] = {
+	{"reduce_t",  LOPT_REDUCE_T},
+	{"bracket", LOPT_BRACKET_PAUSE},
+	{NULL,   -1} };
+
+static MNEM_TAB keyword_tab[] = {
 	{"name",       V_NAME},
 	{"language",   V_LANGUAGE},
 	{"gender",     V_GENDER},
@@ -127,6 +133,7 @@ static keywtab_t keyword_tab[] = {
 	{"stressAmp",  V_STRESSAMP},
 	{"stressAdd",  V_STRESSADD},
 	{"intonation", V_INTONATION},
+	{"tunes",      V_TUNES},
 	{"dictrules",	V_DICTRULES},
 	{"stressrule", V_STRESSRULE},
 	{"charset",    V_CHARSET},
@@ -144,6 +151,9 @@ static keywtab_t keyword_tab[] = {
 	{"option",     V_OPTION},
 	{"mbrola",     V_MBROLA},
 	{"consonants", V_CONSONANTS},
+	{"klatt",      V_KLATT},
+	{"fast_test2",  V_FAST},
+	{"speed",      V_SPEED},
 
 	// these just set a value in langopts.param[]
 	{"l_dieresis", 0x100+LOPT_DIERESES},
@@ -161,11 +171,12 @@ const char variants_male[N_VOICE_VARIANTS] = {1,2,3,4,5,0};
 const char variants_female[N_VOICE_VARIANTS] = {11,12,13,14,0};
 const char *variant_lists[3] = {variants_either, variants_male, variants_female};
 
-voice_t voicedata;
+static voice_t voicedata;
 voice_t *voice = &voicedata;
 
-char *fgets_strip(char *buf, int size, FILE *f_in)
-{//===============================================
+
+static char *fgets_strip(char *buf, int size, FILE *f_in)
+{//======================================================
 // strip trailing spaces, and truncate lines at // comment
 	int len;
 	char *p;
@@ -184,8 +195,22 @@ char *fgets_strip(char *buf, int size, FILE *f_in)
 }
 
 
-void SetToneAdjust(voice_t *voice, int *tone_pts)
-{//==============================================
+static int LookupTune(const char *name)
+{//====================================
+	int ix;
+
+	for(ix=0; ix<n_tunes; ix++)
+	{
+		if(strcmp(name, tunes[ix].name) == 0)
+			return(ix);
+	}
+	return(-1);
+}  // end of LookupTune
+
+
+
+static void SetToneAdjust(voice_t *voice, int *tone_pts)
+{//=====================================================
 	int ix;
 	int pt;
 	int y;
@@ -195,7 +220,7 @@ void SetToneAdjust(voice_t *voice, int *tone_pts)
 	int height2;
 	double rate;
 
-	for(pt=0; pt<10; pt+=2)
+	for(pt=0; pt<12; pt+=2)
 	{
 		if(tone_pts[pt] == -1)
 		{
@@ -225,15 +250,16 @@ void SetToneAdjust(voice_t *voice, int *tone_pts)
 
 void ReadTonePoints(char *string, int *tone_pts)
 {//=============================================
-// tone_pts[] is int[10]
+// tone_pts[] is int[12]
 	int ix;
 
-	for(ix=0; ix<10; ix++)
+	for(ix=0; ix<12; ix++)
 		tone_pts[ix] = -1;
 
-	sscanf(string,"%d %d %d %d %d %d %d %d",
+	sscanf(string,"%d %d %d %d %d %d %d %d %d %d",
 		&tone_pts[0],&tone_pts[1],&tone_pts[2],&tone_pts[3],
-		&tone_pts[4],&tone_pts[5],&tone_pts[6],&tone_pts[7]);
+		&tone_pts[4],&tone_pts[5],&tone_pts[6],&tone_pts[7],
+		&tone_pts[8],&tone_pts[9]);
 }
 
 
@@ -358,21 +384,36 @@ void VoiceReset(int tone_only)
 // Set voice to the default values
 
 	int  pk;
+	static unsigned char default_heights[N_PEAKS] = {128,128,120,120,110,110,128,128,128};
+	static unsigned char default_widths[N_PEAKS] = {128,128,128,160,171,171,128,128,128};
+
 	static int breath_widths[N_PEAKS] = {0,200,200,400,400,400,600,600,600};
 
-	// default is:  pitch 82,118
-	voice->pitch_base =   0x49000;    // default, 73 << 12;
-	voice->pitch_range =  0x0f30;     // default = 0x1000
+	// default is:  pitch 80,118
+	voice->pitch_base = 0x47000;
+	voice->pitch_range = 4104;
+
+//	default is:  pitch 80,117
+//	voice->pitch_base = 0x47000;
+//	voice->pitch_range = 3996;
+
 	voice->formant_factor = 256;
 
+	voice->speed_percent = 100;
 	voice->echo_delay = 0;
 	voice->echo_amp = 0;
 	voice->flutter = 64;
 	voice->n_harmonic_peaks = 5;
-	voice->peak_shape = 1;
+	voice->peak_shape = 0;
 	voice->voicing = 64;
 	voice->consonant_amp = 100;
 	voice->consonant_ampv = 100;
+	voice->samplerate = 22050;
+	memset(voice->klattv,0,sizeof(voice->klattv));
+
+	speed.fast_settings[0] = 400;
+	speed.fast_settings[1] = 800;
+	speed.fast_settings[2] = 175;
 
 #ifdef PLATFORM_RISCOS
 	voice->roughness = 1;
@@ -384,8 +425,8 @@ void VoiceReset(int tone_only)
 	for(pk=0; pk<N_PEAKS; pk++)
 	{
 		voice->freq[pk] = 256;
-		voice->height[pk] = 256;
-		voice->width[pk] = 256;
+		voice->height[pk] = default_heights[pk]*2;
+		voice->width[pk] = default_widths[pk]*2;
 		voice->breath[pk] = 0;
 		voice->breathw[pk] = breath_widths[pk];  // default breath formant woidths
 		voice->freqadd[pk] = 0;
@@ -409,7 +450,6 @@ SetToneAdjust(voice,tone_points);
 	if(tone_only == 0)
 	{
 		n_replace_phonemes = 0;
-		option_tone1 = 0;
 		option_quiet = 0;
 		LoadMbrolaTable(NULL,NULL,0);
 	}
@@ -459,11 +499,11 @@ static void PhonemeReplacement(int type, char *p)
 	if((n < 2) || (n_replace_phonemes >= N_REPLACE_PHONEMES))
 		return;
 
-	if((phon = LookupPh(phon_string1)) == 0)
+	if((phon = LookupPhonemeString(phon_string1)) == 0)
 		return;  // not recognised
 
 	replace_phonemes[n_replace_phonemes].old_ph = phon;
-	replace_phonemes[n_replace_phonemes].new_ph = LookupPh(phon_string2);
+	replace_phonemes[n_replace_phonemes].new_ph = LookupPhonemeString(phon_string2);
 	replace_phonemes[n_replace_phonemes++].type = flags;
 }  //  end of PhonemeReplacement
 
@@ -472,6 +512,7 @@ static void PhonemeReplacement(int type, char *p)
 static int Read8Numbers(char *data_in,int *data)
 {//=============================================
 // Read 8 integer numbers
+	memset(data, 0, 8+sizeof(int));
 	return(sscanf(data_in,"%d %d %d %d %d %d %d %d",
 		&data[0],&data[1],&data[2],&data[3],&data[4],&data[5],&data[6],&data[7]));
 }
@@ -485,12 +526,12 @@ voice_t *LoadVoice(const char *vname, int control)
 //          bit 4  1 = vname = full path
 
 	FILE *f_voice = NULL;
-	keywtab_t *k;
 	char *p;
 	int  key;
 	int  ix;
 	int  n;
 	int  value;
+	int  value2;
 	int  error = 0;
 	int  langix = 0;
 	int  tone_only = control & 2;
@@ -509,6 +550,7 @@ voice_t *LoadVoice(const char *vname, int control)
 	char translator_name[40];
 	char new_dictionary[40];
 	char phonemes_name[40];
+	char option_name[40];
 	const char *language_type;
 	char buf[200];
 	char path_voices[sizeof(path_home)+12];
@@ -517,6 +559,7 @@ voice_t *LoadVoice(const char *vname, int control)
 	int stress_amps[8];
 	int stress_lengths[8];
 	int stress_add[8];
+	char names[8][40];
 
 	int pitch1;
 	int pitch2;
@@ -547,6 +590,18 @@ voice_t *LoadVoice(const char *vname, int control)
 			langname[1] = voicename[1];
 			langname[2] = 0;
 			sprintf(buf,"%s%s%c%s",path_voices,langname,PATHSEP,voicename);
+
+			if(GetFileLength(buf) <= 0)
+			{
+				// look in "extra" sub-directory
+				sprintf(buf,"%sextra%c%s",path_voices,PATHSEP,voicename);
+
+				if(GetFileLength(buf) <= 0)
+				{
+					// look in "test" sub-directory
+					sprintf(buf,"%stest%c%s",path_voices,PATHSEP,voicename);
+				}
+			}
 		}
 	}
 
@@ -564,7 +619,7 @@ voice_t *LoadVoice(const char *vname, int control)
 
 	if(!tone_only && (translator != NULL))
 	{
-		delete translator;
+		DeleteTranslator(translator);
 		translator = NULL;
 	}
 
@@ -607,15 +662,7 @@ voice_t *LoadVoice(const char *vname, int control)
 
 		if(buf[0] == 0) continue;
 
-		key = 0;
-		for(k=keyword_tab; k->mnem != NULL; k++)
-		{
-			if(strcmp(buf,k->mnem)==0)
-			{
-				key = k->data;
-				break;
-			}
-		}
+		key = LookupMnem(keyword_tab, buf);
 
 		switch(key)
 		{
@@ -655,7 +702,7 @@ voice_t *LoadVoice(const char *vname, int control)
 					SelectPhonemeTableName(phonemes_name);
 		
 					if(new_translator != NULL)
-							delete new_translator;
+							DeleteTranslator(new_translator);
 		
 					new_translator = SelectTranslator(translator_name);
 					langopts = &new_translator->langopts;
@@ -687,7 +734,7 @@ voice_t *LoadVoice(const char *vname, int control)
 			sscanf(p,"%s",translator_name);
 
 			if(new_translator != NULL)
-					delete new_translator;
+					DeleteTranslator(new_translator);
 
 			new_translator = SelectTranslator(translator_name);
 			langopts = &new_translator->langopts;
@@ -735,6 +782,21 @@ voice_t *LoadVoice(const char *vname, int control)
 				langopts->intonation_group = option_tone_flags & 0xff;
 			break;
 
+		case V_TUNES:
+			n = sscanf(p,"%s %s %s %s %s %s",names[0],names[1],names[2],names[3],names[4],names[5]);
+			langopts->intonation_group = 0;
+			for(ix=0; ix<n; ix++)
+			{
+				if(strcmp(names[ix],"NULL")==0)
+					continue;
+
+				if((value = LookupTune(names[ix])) < 0)
+					fprintf(stderr,"Unknown tune '%s'\n",names[ix]);
+				else
+					langopts->tunes[ix] = value;
+			}
+			break;
+
 		case V_DICTRULES:   // conditional dictionary rules and list entries
 			while(*p != 0)
 			{
@@ -776,14 +838,19 @@ voice_t *LoadVoice(const char *vname, int control)
 			break;
 
 		case V_NUMBERS:
-			sscanf(p,"%d",&langopts->numbers);
+			sscanf(p,"%d %d",&langopts->numbers,&langopts->numbers2);
 			break;
 
 		case V_OPTION:
-			if(sscanf(p,"%d %d",&ix,&value) == 2)
+			value2 = 0;
+			if((sscanf(p,"%s %d %d",option_name,&value,&value2) >= 2) && ((ix = LookupMnem(options_tab, option_name)) >= 0))
 			{
-				if((ix >= 0) && (ix < N_LOPTS))
-					langopts->param[ix] = value;
+				langopts->param[ix] = value;
+				langopts->param2[ix] = value2;
+			}
+			else
+			{
+				fprintf(stderr,"Bad voice option: %s %s\n",buf,p);
 			}
 			break;
 
@@ -818,7 +885,7 @@ voice_t *LoadVoice(const char *vname, int control)
 
 		case V_TONE:
 			{
-				int tone_data[10];
+				int tone_data[12];
 				ReadTonePoints(p,tone_data);
 				SetToneAdjust(voice,tone_data);
 			}
@@ -846,6 +913,10 @@ voice_t *LoadVoice(const char *vname, int control)
 			value = sscanf(p,"%d %d",&voice->consonant_amp, &voice->consonant_ampv);
 			break;
 
+		case V_SPEED:
+			sscanf(p,"%d",&voice->speed_percent);
+			break;
+
 		case V_MBROLA:
 			{
 				int srate = 16000;
@@ -854,10 +925,25 @@ voice_t *LoadVoice(const char *vname, int control)
 
 				phtrans[0] = 0;
 				sscanf(p,"%s %s %d",name,phtrans,&srate);
-				LoadMbrolaTable(name,phtrans,srate);
+				if(LoadMbrolaTable(name,phtrans,srate) != EE_OK)
+				{
+					fprintf(stderr,"mbrola voice not found\n");
+				}
+				voice->samplerate = srate;
 			}
 			break;
 
+		case V_KLATT:
+			voice->klattv[0] = 1;  // default source: IMPULSIVE
+			Read8Numbers(p,voice->klattv);
+			voice->klattv[KLATT_Kopen] -= 40;
+			break;
+
+		case V_FAST:
+			Read8Numbers(p,speed.fast_settings);
+			SetSpeed(3);
+			break;
+
 		default:
 			if((key & 0xff00) == 0x100)
 			{
@@ -879,6 +965,8 @@ voice_t *LoadVoice(const char *vname, int control)
 		new_translator = SelectTranslator(translator_name);
 	}
 
+	SetSpeed(3);   // for speed_percent
+
 	for(ix=0; ix<N_PEAKS; ix++)
 	{
 		voice->freq2[ix] = voice->freq[ix];
@@ -897,7 +985,8 @@ voice_t *LoadVoice(const char *vname, int control)
 			fprintf(stderr,"Unknown phoneme table: '%s'\n",phonemes_name);
 		}
 		voice->phoneme_tab_ix = ix;
-		error = new_translator->LoadDictionary(new_dictionary, control & 4);
+		new_translator->phoneme_tab_ix = ix;
+		error = LoadDictionary(new_translator, new_dictionary, control & 4);
 		if(dictionary_name[0]==0)
 			return(NULL);   // no dictionary loaded
 
@@ -940,8 +1029,8 @@ voice_t *LoadVoice(const char *vname, int control)
 }  //  end of LoadVoice
 
 
-char *ExtractVoiceVariantName(char *vname, int variant_num)
-{//========================================================
+static char *ExtractVoiceVariantName(char *vname, int variant_num)
+{//===============================================================
 // Remove any voice variant suffix (name or number) from a voice name
 // Returns the voice variant name
 
@@ -1106,7 +1195,7 @@ static int ScoreVoice(espeak_VOICE *voice_spec, const char *spec_language, int s
 			matching_parts += matching;  // number of parts which match
 
 			if(matching_parts == 0)
-				break;   // no matching parts for this language
+				continue;   // no matching parts for this language
 
 			x = 5;
 			// reduce the score if not all parts of the required language match
@@ -1285,8 +1374,8 @@ espeak_VOICE *SelectVoiceByName(espeak_VOICE **voices, const char *name)
 
 
 
-char const *SelectVoice(espeak_VOICE *voice_select)
-{//================================================
+char const *SelectVoice(espeak_VOICE *voice_select, int *found)
+{//============================================================
 // Returns a path within espeak-voices, with a possible +variant suffix
 // variant is an output-only parameter
 	int nv;           // number of candidates
@@ -1307,6 +1396,7 @@ char const *SelectVoice(espeak_VOICE *voice_select)
 	static espeak_VOICE voice_variants[N_VOICE_VARIANTS];
 	static char voice_id[50];
 
+	*found = 1;
 	memcpy(&voice_select2,voice_select,sizeof(voice_select2));
 
 	if(n_voices_list == 0)
@@ -1350,6 +1440,7 @@ char const *SelectVoice(espeak_VOICE *voice_select)
 	if(nv == 0)
 	{
 		// no matching voice, choose the default
+		*found = 0;
 		if((voices[0] = SelectVoiceByName(voices_list,"default")) != NULL)
 			nv = 1;
 	}
@@ -1415,6 +1506,8 @@ char const *SelectVoice(espeak_VOICE *voice_select)
 	}
 
 	// index the sorted list by the required variant number
+	if(ix2 == 0)
+		return(NULL);
 	vp = voices2[voice_select2.variant % ix2];
 
 	if(vp->variant != 0)
@@ -1429,8 +1522,8 @@ char const *SelectVoice(espeak_VOICE *voice_select)
 
 
 
-void GetVoices(const char *path)
-{//=============================
+static void GetVoices(const char *path)
+{//====================================
 	FILE *f_voice;
 	espeak_VOICE *voice_data;
 	int ftype;
@@ -1487,13 +1580,12 @@ void GetVoices(const char *path)
 	}
 #else
 #ifdef PLATFORM_WINDOWS
-   WIN32_FIND_DATA FindFileData;
+   WIN32_FIND_DATAA FindFileData;
    HANDLE hFind = INVALID_HANDLE_VALUE;
-   DWORD dwError;
 
 #undef UNICODE         // we need FindFirstFileA() which takes an 8-bit c-string
 	sprintf(fname,"%s\\*",path);
-	hFind = FindFirstFile(fname, &FindFileData);
+	hFind = FindFirstFileA(fname, &FindFileData);
 	if(hFind == INVALID_HANDLE_VALUE)
 		return;
 
@@ -1523,14 +1615,14 @@ void GetVoices(const char *path)
 				voices_list[n_voices_list++] = voice_data;
 			}
 		}
-	} while(FindNextFile(hFind, &FindFileData) != 0);
+	} while(FindNextFileA(hFind, &FindFileData) != 0);
 	FindClose(hFind);
 
 #else
 	DIR *dir;
 	struct dirent *ent;
 
-	if((dir = opendir(path)) == NULL)
+	if((dir = opendir((char *)path)) == NULL)    // note: (char *) is needed for WINCE
 		return;
 
 	while((ent = readdir(dir)) != NULL)
@@ -1624,8 +1716,12 @@ espeak_ERROR SetVoiceByName(const char *name)
 espeak_ERROR SetVoiceByProperties(espeak_VOICE *voice_selector)
 {//============================================================
 	const char *voice_id;
+	int voice_found;
 
-	voice_id = SelectVoice(voice_selector);
+	voice_id = SelectVoice(voice_selector, &voice_found);
+
+	if(voice_found == 0)
+		return(EE_NOT_FOUND);
 
 	LoadVoiceVariant(voice_id,0);
 	DoVoiceChange(voice);
